package com.qianmi.autoupdater;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.util.Log;

import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.ShellUtils;
import com.blankj.utilcode.util.Utils;
import com.facebook.react.ReactActivity;
import com.facebook.react.ReactApplication;
import com.facebook.react.ReactNativeHost;

import top.wuhaojie.installerlibrary.AutoInstaller;

import static com.qianmi.autoupdater.RNRenderModule.BUNDLE_APK_UPDATER_RENDER_ACTION;
import static com.qianmi.autoupdater.RNRenderModule.BUNDLE_OR_APP_URI;
import static com.qianmi.autoupdater.UpdaterUriType.APK;
import static com.qianmi.autoupdater.UpdaterUriType.BUNDLE;
import static com.qianmi.autoupdater.UpdaterUriType.JS;


/**
 * Created by xiayinglin on 2017/9/24.
 */
public class BundleOrApkUpdateReceiver extends BroadcastReceiver {


    private Activity mActivity;


    public BundleOrApkUpdateReceiver(Activity activity) {
        this.mActivity = activity;
        Utils.init(activity.getApplication());
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        Log.d(getClass().getSimpleName(), "=== BundleOrApkUpdateReceiver.onReceive ===");
        if (null == mActivity) {
            Log.w(getClass().getSimpleName(), "BundleOrApkUpdateReceiver.onReceive mActivity isNull!");
            return;
        }


        final ReactNativeHost host =
                ((ReactApplication) mActivity.getApplication())
                        .getReactNativeHost();

        //判断是否为更新
        if (BUNDLE_APK_UPDATER_RENDER_ACTION.equals(intent.getAction())) {
            String uri = intent.getStringExtra(BUNDLE_OR_APP_URI);
            Log.d(getClass().getSimpleName(), "BundleOrApkUpdateReceiver.onReceive uri:" + uri);
            if (TextUtils.isEmpty(uri)) {
                return;
            }

            if (uri.endsWith(APK.getType())) {
//                try {
//                    //首先静默升级，升级失败执行手动升级（需要用户在界面上点确认）
//                    if (AppUtils.isAppRoot()) {
//                        AppUtils.installAppSilent(uri);
//                    } else {
//                        AppUtils.installApp(uri, "com.qianmi.fileprovider");
//                    }
//                } catch (Exception e) {
//                    Log.e(getClass().getSimpleName(), e.toString());
//                    AppUtils.installApp(uri, null);
//                }
                //主要就是通过,安装程序之前,启动一个定时任务,任务发送一个广播,广播收到之后,启动程序
                AutoInstaller.getDefault(mActivity).install(uri);
            } else if (uri.endsWith(BUNDLE.getType()) || uri.endsWith(JS.getType())) {
                host.clear();
                if (mActivity instanceof ReactActivity) {
                    ReactActivity rca = ((ReactActivity) mActivity);
                    Log.d(getClass().getSimpleName(), "recreate");
                    rca.recreate();
                }
            }
        }
    }


    /**
     * 安装后自启动apk
     *
     * @param context
     */
    private void startApp(String packageName, Context context) {
        Log.d(getClass().getSimpleName(), "BundleOrApkUpdateReceiver.startApp");
        ShellUtils.execCmd("am start -S " + context.getPackageName() + "/"
                + packageName + " \n", AppUtils.isSystemApp());
    }


}
