package com.qianmi.autoupdater;

import android.content.BroadcastReceiver;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Environment;
import android.text.TextUtils;
import android.util.Log;

import com.blankj.utilcode.util.FileIOUtils;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by xiayinglin on 2017/9/25.
 */

public class RNRenderModule extends ReactContextBaseJavaModule {


    public final static String BUNDLE_APK_URI_STORAGE_KEY = "rn_render_storage_key";

    public final static String OBJECT_KEY_BUNDLE_FILE = "bundle_file";

    public final static String OBJECT_KEY_BUNDLE_VERSION = "bundle_version";
    public final static String OBJECT_KEY_APP_VERSION = "app_version";
    public final static String OBJECT_KEY_APP_FILE = "app_file";


    //重新加载bundle or apk
    public final static String BUNDLE_APK_UPDATER_RENDER_ACTION = "com.auto_updater.render";
    public final static String BUNDLE_OR_APP_URI = "updater_uri";


    public RNRenderModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }

    private BroadcastReceiver bundleOrApkUpdateReceiver = null;


    public Map<String, String> getVersion() {
        File externalStorageDirectory = Environment.getExternalStorageDirectory();
        String path = externalStorageDirectory != null ? externalStorageDirectory.getAbsolutePath()
                : getReactApplicationContext().getFilesDir().getAbsolutePath();
        return toData(FileIOUtils.readFile2String(new File(path, "version.json")));
    }

    /**
     * 通知刷新UI
     */
    @ReactMethod
    public void render() {
        init();
//        ReactDatabaseSupplier mReactDataSupplier = ReactDatabaseSupplier.getInstance(getReactApplicationContext());
//        String data = AsyncLocalStorageUtil.getItemImpl(mReactDataSupplier.get(), BUNDLE_APK_URI_STORAGE_KEY);
//        Log.d(getClass().getSimpleName(), "---->" + data);
//        Intent intent = new Intent(BUNDLE_APK_UPDATER_RENDER_ACTION);
//        Map<String, String> d = toData(data);

        Intent intent = new Intent(BUNDLE_APK_UPDATER_RENDER_ACTION);
        Map<String, String> d = getVersion();
        String appUri = d.get(OBJECT_KEY_APP_FILE);
        if (!TextUtils.isEmpty(appUri)) {
            intent.putExtra(BUNDLE_OR_APP_URI, appUri);
        } else {
            intent.putExtra(BUNDLE_OR_APP_URI, d.get(OBJECT_KEY_BUNDLE_FILE));
        }
        getReactApplicationContext().sendBroadcast(intent);
    }

    /**
     * 初始化升级广播
     */
    private void init() {
        if (null != bundleOrApkUpdateReceiver) {
            getReactApplicationContext().unregisterReceiver(bundleOrApkUpdateReceiver);
        }

        bundleOrApkUpdateReceiver = new BundleOrApkUpdateReceiver(getCurrentActivity());
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BUNDLE_APK_UPDATER_RENDER_ACTION);
        getReactApplicationContext().registerReceiver(bundleOrApkUpdateReceiver, intentFilter);
        Log.d(getClass().getSimpleName(), "BundleOrApkUpdateReceiver register!");
    }

    @Override
    public String getName() {
        return "RNRender";
    }

    public static Map<String, String> toData(String data) {
        if (TextUtils.isEmpty(data)) {
            return new HashMap<>();
        } else {
            GsonBuilder gb = new GsonBuilder();
            Gson g = gb.create();
            return g.fromJson(data, new TypeToken<Map<String, String>>() {
            }.getType());
        }
    }
}
