import { NetInfo, AsyncStorage, Platform } from 'react-native';

import RNFS from 'react-native-fs';

export default {
    tofileName,
    getJSON,
    download
}


/**
 * 生成名称
 * @param {*} url 
 */
function tofileName(url) {
    if (__DEV__) console.log(RNFS);
    var name = `${RNFS.DocumentDirectoryPath}/${url.substring(url.lastIndexOf('/'), url.length)}`;
    if ('android' === Platform.OS) {
        name = `${RNFS.ExternalStorageDirectoryPath}/${url.substring(url.lastIndexOf('/'), url.length)}`;
    }
    return name;
}

/**
 *  千米ares 接口
 * @param {*} url 
 */
function getJSON(url) {
    return new Promise((resolve, reject) => {
        fetch(url, {
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json',
            },
            method: 'GET',
        })
            .then((response) => {
                if (__DEV__) console.log(response);
                resolve(response.json());
            })
            .catch((error) => {
                if (__DEV__) console.error(error);
                reject(error);
            });
    })

}

/**
 * 下载文件到本地
 * @param {*} url 
 * @param {*} toFile 
 * @param {*} progressFunc 
 */
function download(url, toFile, progressFunc) {
    return new Promise((resolve, reject) => {
        const options = {
            fromUrl: url,
            toFile: toFile,
            background: true,
            begin: (res) => {
                if (__DEV__) console.log('begin', res);
                // if (__DEV__) console.log('contentLength:', res.contentLength / 1024 / 1024, 'M');
            },
            progress: (res) => {
                let pro = res.bytesWritten / res.contentLength;
                if (progressFunc) progressFunc(pro);
                // if (__DEV__) console.log(pro);
            }
        };
        try {
            const ret = RNFS.downloadFile(options);
            ret.promise.then(res => {
                if (__DEV__) console.log('success', res);
                if (__DEV__) console.log(`toFile : ${toFile}`);
                resolve({
                    url: url,
                    toFile: toFile,
                });
            }).catch(error => {
                if (__DEV__) console.log('err', error);
                reject(error);
            });
        }
        catch (error) {
            if (__DEV__) console.log(error);
            reject(error);
        }
    })
}
