import { NetInfo, AsyncStorage, Platform } from 'react-native';

import Download from './download';
import RNFS from 'react-native-fs';


export default {
    Upgrade_App,
    Upgrade_Bundle
}


/**
 * info.pid, info.platform, info.versionCode
 * @param {*} info 
 * @param {*} progressFunc 
 */
function Upgrade_App(info, progressFunc) {
    return new Promise((resolve, reject) => {
       getVersion().then(r => {
        var data = JSON.parse(r | '{}');
            app_ares(info.pid, info.platform, info.versionCode).then(result => {
                if (null == result) {
                    resolve(false);
                    return;
                }
                data['app_version'] = result['versionCode'];
                data['app_file'] = result['toFile'];

                //当前app版本大于等于接口获取版本，则不需要更新处理
                if (info.versionCode >= data['app_version']) {
                    resolve(false);
                    // return new Promise((resolve, reject) => { resolve(false) });
                } else {
                    return Download.download(result['url'], result['toFile'], progressFunc);
                }
            }).then(result => {
                // AsyncStorage.setItem('rn_render_storage_key', JSON.stringify(data), err => { resolve(err ? false : true); });
                toVersion(data).then(r => {
                    resolve(r ? true : false); 
                });
            }).catch(e => {
                if (__DEV__) console.error(e);
                reject(e);
            });
            
        });
        
    });
}

/**
 * info.pid, info.eid, info.platform, info.sv ,info.version
 * @param {*} info 
 * @param {*} progressFunc 
 */
function Upgrade_Bundle(info, progressFunc) {
    let bundleVersionKey = `rn_bundle_${info.pid}_${info.eid}_${info.platform}_${info.sv}`;
    
    console.log('info ===>', info);
    return new Promise((resolve, reject) => {
        getVersion().then(r => {
            var local = JSON.parse(r | '{}');
            // var bunderUnique = null;
            bundle_ares(info.pid, info.eid, info.platform, info.sv).then(result => {
                //版本获取失败或相同不做更新
                if (null == result 
                    || info.version == result['version']) {
                    console.log('bundle no update', result);                        
                    resolve(false);
                    return;
                }

                return new Promise((resolve, reject) => {
                    temp['bundle_version'] = result['version'];
                    temp['bundle_file'] = result['toFile'];
                    
                    temp['url'] = result['url'];
                    resolve(temp);
                });
            }).then(remote => {

                //对比版本信息，判断是否需要下载
                if (remote['bundle_version'] != local['bundle_version']) {

                    local['bundle_version'] = remote['bundle_version'];
                    local['bundle_file'] = remote['bundle_file'];
                    
                    //bundle version + url 如果不等 需要重新加载接口中的版本
                    return Download.download(remote['url'], remote['bundle_file'], progressFunc);
                } else {
                    return new Promise((resolve, reject) => {
                         resolve(false);
                    });
                }
            }).then(result => {
                if (result) {
                    toVersion(local).then(r => {
                        resolve(r ? true : false); 
                    });
                } else {
                    resolve(false);
                }
            }).catch(e => {
                if (__DEV__) console.error(e);
                reject(e);
            });
        });

       
    });
}


/**
 * 
 * @param {*} pid 
 * @param {*} platform 
 * @param {*} versionCode 
 */
function app_ares(pid, platform, versionCode) {
    //'http://aresapi.qianmi.com/api/app/version/check?pid=8&platform=a&versionCode=3'
    return app(`http://aresapi.qianmi.com/api/app/version/check?pid=${pid}&platform=${platform}&versionCode=${versionCode}`);
}


/**
 * app升级接口
 */
function app(url) {
    return new Promise((resolve, reject) => {
        Download.getJSON(url).then(response => {
            if (1 != response.status) {
                reject(response.message);
            } else {
                let data = response.data;
                let needforceUpdate = data['needforceUpdate'];
                let lastestVersion = data['lastestVersion'];
                if (null == lastestVersion) {
                    resolve(null);
                    return;
                }
                let projectId = lastestVersion['project']['id'];
                let url = lastestVersion['apkUrl'];
                let addDate = lastestVersion['addDate'];
                let versionCode = lastestVersion['versionCode'];
                var name = Download.tofileName(url);
                resolve({
                    needforceUpdate: needforceUpdate,
                    url: url,
                    versionCode: versionCode,
                    toFile: name,
                    id: projectId,
                    addDate: addDate
                });
            }
        }).catch((error) => {
            if (__DEV__) console.error(error);
            reject(error);
        });
    });
}

/**
 * 
 * @param {*} pid 
 * @param {*} eid 
 * @param {*} platform 
 * @param {*} sv 
 */
function bundle_ares(pid, eid, platform, sv) {
    //http://aresapi.qianmi.com/api/map?pid=8&eid=11&platform=android&sv=1
    return bundle(`http://aresapi.qianmi.com/api/map?pid=${pid}&eid=${eid}&platform=${platform}&sv=${sv}`);
}

/**
 * bundle新版本查询接口
 */
function bundle(url) {
    return new Promise((resolve, reject) => {
        Download.getJSON(url).then(response => {
            if (1 != response.status) {
                reject(response.message);
            } else {
                if (0 === response.data['routes'].length) {
                    resolve(null);
                } else {
                    let route = response.data['routes'][0];
                    resolve({
                        url: route['remote_file'],
                        toFile: Download.tofileName(url),
                        version: route['version']
                    });
                }
            }
        }).catch((error) => {
            if (__DEV__) console.error(error);
            reject(error);
        });
    });
}


function getVersion() {
    RNFS.exists(getVersionFile()).then(r => {
        if(r){
            return RNFS.readFile(getVersionFile(), 'utf8');
        } else{
            return RNFS.writeFile('{ "app_version" : "" , "app_file" : "", "bundle_file" : "", "bundle_version" : "" }', , 'utf8')
            .then((success) => {
                return RNFS.readFile(getVersionFile(), 'utf8');
            }).catch((err) => {
                if(__DEV__) console.error(err.message);
                return new Promise((resolve, reject) => {
                    resolve(null);
                });
            });
        }
    });
}

function toVersion(data){
    return RNFS.writeFile(data, , 'utf8').then((success) => {
        return RNFS.readFile(getVersionFile(), 'utf8');
    }).catch((err) => {
        if(__DEV__) console.error(err.message);
        return new Promise((resolve, reject) => {
            resolve(null);
        });
    });
}


function path(){
    return 'android' === Platform.OS ? RNFS.ExternalStorageDirectoryPath : RNFS.DocumentDirectoryPath
}

function getVersionFile() {
    return `${path()}/version.json`
}